@@ -3,7 +3,7 @@
  * gog-style.c :
  *
  * Copyright (C) 2003-2004 Jody Goldberg (jody@gnome.org)
- * Copyright (C) 2006      Emmanuel Pacaud (emmanuel.pacaud@lapp.in2p3.fr)
+ * Copyright (C) 2006-2007 Emmanuel Pacaud (emmanuel.pacaud@lapp.in2p3.fr)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -23,16 +23,17 @@
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-styled-object.h>
+#include <goffice/math/go-math.h>
 #include <goffice/utils/go-color.h>
 #include <goffice/utils/go-font.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/utils/go-image.h>
 #include <goffice/utils/go-line.h>
 #include <goffice/utils/go-marker.h>
-#include <goffice/utils/go-math.h>
 
 #ifdef GOFFICE_WITH_GTK
 #include <goffice/gtk/goffice-gtk.h>
+#include <goffice/gtk/go-pixbuf.h>
 #include <goffice/gtk/go-font-sel.h>
 #include <goffice/gtk/go-palette.h>
 #include <goffice/gtk/go-rotation-sel.h>
@@ -96,6 +97,7 @@ typedef struct {
 		GtkWidget *foreground_box;
 		GtkWidget *foreground_label;
 		GtkWidget *notebook;
+		GtkWidget *extension_box;
 		struct {
 			GtkWidget *selector;
 			GtkWidget *box;
@@ -217,17 +219,20 @@ cb_outline_color_changed (GOSelector *selector,
 }
 
 static void
-outline_init (StylePrefState *state, gboolean enable)
+outline_init (StylePrefState *state, gboolean enable, GogEditor *editor)
 {
 	GogStyle *style = state->style;
 	GogStyle *default_style = state->default_style;
 	GtkWidget *w, *table;
 
+	w = glade_xml_get_widget (state->gui, "outline_box");
 	if (!enable) {
-		gtk_widget_hide (glade_xml_get_widget (state->gui, "outline_box"));
+		gtk_widget_hide (w);
 		return;
 	}
 
+	gog_editor_register_widget (editor, w);
+
 	table = glade_xml_get_widget (state->gui, "outline_table");
 
 	/* DashType */
@@ -288,17 +293,20 @@ cb_line_color_changed (GOSelector *selector,
 }
 
 static void
-line_init (StylePrefState *state, gboolean enable)
+line_init (StylePrefState *state, gboolean enable, GogEditor *editor)
 {
 	GogStyle *style = state->style;
 	GogStyle *default_style = state->default_style;
 	GtkWidget *w, *table;
 
+	w = glade_xml_get_widget (state->gui, "line_box");
 	if (!enable) {
-		gtk_widget_hide (glade_xml_get_widget (state->gui, "line_box"));
+		gtk_widget_hide (w);
 		return;
 	}
 
+	gog_editor_register_widget (editor, w);
+
 	table = glade_xml_get_widget (state->gui, "line_table");
 
 	/* DashType */
@@ -325,43 +333,6 @@ line_init (StylePrefState *state, gboolean enable)
 	gtk_widget_show_all (table);
 }
 
-
-/************************************************************************/
-
-static void
-cb_line_interpolation_changed (GtkComboBox *box, StylePrefState const *state)
-{
-	GogStyle *style = state->style;
-
-	g_return_if_fail (style != NULL);
-
-	style->interpolation.type = gtk_combo_box_get_active (box);
-	style->interpolation.auto_type = FALSE;
-	set_style (state);
-}
-
-static void
-interp_init (StylePrefState *state, gboolean enable)
-{
- 	GogStyle *style = state->style;
-	GtkWidget *w;
-
-	if (enable) {
- 		w = glade_xml_get_widget (state->gui, "interpolation-box");
- 		gtk_combo_box_set_active (GTK_COMBO_BOX (w), style->interpolation.type);
- 		g_signal_connect (G_OBJECT (w),
- 				  "changed",
- 				  G_CALLBACK (cb_line_interpolation_changed), state);
- 	} else {
- 		w = glade_xml_get_widget (state->gui, "interpolation-lbl");
- 		gtk_widget_hide (w);
- 		w = glade_xml_get_widget (state->gui, "interp-hbox");
- 		gtk_widget_hide (w);
- 	}
-}
-
-/************************************************************************/
-
 static void cb_fill_background_color (GOSelector *selector, StylePrefState *state);
 static void cb_fill_foreground_color (GOSelector *selector, StylePrefState *state);
 
@@ -648,11 +619,12 @@ static void
 fill_update_visibilies (FillType type, StylePrefState *state)
 {
 	g_object_set (state->fill.pattern.box, "visible" , fill_infos[type].show_pattern, NULL);
-	g_object_set (state->fill.gradient.box, "visible", fill_infos[type].show_gradient, NULL);	
-	g_object_set (state->fill.gradient.brightness_box, "visible", 
+	g_object_set (state->fill.gradient.box, "visible", fill_infos[type].show_gradient, NULL);
+	g_object_set (state->fill.gradient.brightness_box, "visible",
 		      fill_infos[type].show_brightness, NULL);
-	g_object_set (state->fill.foreground_box, "visible", 
+	g_object_set (state->fill.foreground_box, "visible",
 		      !fill_infos[type].show_brightness, NULL);
+	g_object_set (state->fill.extension_box, "visible", type != FILL_TYPE_NONE, NULL);
 
 	if (fill_infos[type].show_gradient) {
 		gtk_label_set_text (GTK_LABEL (state->fill.foreground_label), _("Start:"));
@@ -673,13 +645,14 @@ cb_fill_type_changed (GtkWidget *menu, StylePrefState *state)
 	index = CLAMP (gtk_combo_box_get_active (GTK_COMBO_BOX (menu)), 0, (int) G_N_ELEMENTS (fill_infos) - 1);
 
 	state->style->fill.type = fill_infos[index].type;
+	state->style->fill.auto_type = FALSE;
 	set_style (state);
 
 	fill_update_visibilies (index, state);
 }
 
 static void
-fill_init (StylePrefState *state, gboolean enable)
+fill_init (StylePrefState *state, gboolean enable, GogEditor *editor)
 {
 	GtkWidget *w;
 	FillType type;
@@ -689,6 +662,9 @@ fill_init (StylePrefState *state, gboolean enable)
 		return;
 	}
 
+	state->fill.extension_box = glade_xml_get_widget (state->gui, "fill_extension_box");
+	gog_editor_register_widget (editor, state->fill.extension_box);
+
 	state->fill.size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
 	fill_color_init (state);
@@ -698,7 +674,7 @@ fill_init (StylePrefState *state, gboolean enable)
 	fill_update_selectors (state);
 
 	state->fill.notebook = glade_xml_get_widget (state->gui, "fill_notebook");
-	
+
 	switch (state->style->fill.type) {
 		case GOG_FILL_STYLE_PATTERN:
 			type = FILL_TYPE_PATTERN;
@@ -788,19 +764,22 @@ cb_marker_size_changed (GtkAdjustment *adj, StylePrefState *state)
 }
 
 static void
-marker_init (StylePrefState *state, gboolean enable)
+marker_init (StylePrefState *state, gboolean enable, GogEditor *editor, GOCmdContext *cc)
 {
 	GogStyle *style = state->style;
 	GogStyle *default_style = state->default_style;
 	GtkWidget *table, *w;
 	GtkWidget *selector;
+	GladeXML *gui;
 
-	if (!enable) {
-		gtk_widget_hide (glade_xml_get_widget (state->gui, "marker_box"));
+	if (!enable)
 		return;
-	}
 
-	table = glade_xml_get_widget (state->gui, "marker_table");
+	gui = go_libglade_new ("gog-style-prefs.glade", "gog_style_marker_prefs", GETTEXT_PACKAGE, cc);
+	if (gui == NULL)
+		return;
+
+	table = glade_xml_get_widget (gui, "marker_table");
 
 	state->marker.selector = selector = 
 		go_marker_selector_new (go_marker_get_shape (style->marker.mark),
@@ -812,7 +791,7 @@ marker_init (StylePrefState *state, gboolean enable)
 	else
 		go_marker_selector_set_colors (GO_SELECTOR (selector), 
 					       RGBA_BLUE, RGBA_BLUE);
-	w = glade_xml_get_widget (state->gui, "marker_shape_label");
+	w = glade_xml_get_widget (gui, "marker_shape_label");
 	gtk_label_set_mnemonic_widget (GTK_LABEL (w), selector);
 	gtk_table_attach (GTK_TABLE (table), selector, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	g_signal_connect (G_OBJECT (selector), "activate", 
@@ -823,14 +802,12 @@ marker_init (StylePrefState *state, gboolean enable)
 		w = create_go_combo_color (state,
 			go_marker_get_fill_color (style->marker.mark),
 			go_marker_get_fill_color (default_style->marker.mark),
-			state->gui,
-			"pattern_background", "marker_fill_label",
+			gui, "pattern_background", "marker_fill_label",
 			G_CALLBACK (cb_marker_fill_color_changed));
 	else {
 		w = create_go_combo_color (state,
 			RGBA_BLUE, RGBA_BLUE,
-			state->gui,
-			"pattern_background", "marker_fill_label",
+			gui, "pattern_background", "marker_fill_label",
 			G_CALLBACK (cb_marker_fill_color_changed));
 		gtk_widget_set_sensitive (w, FALSE);
 	}
@@ -840,20 +817,18 @@ marker_init (StylePrefState *state, gboolean enable)
 		w = create_go_combo_color (state,
 			go_marker_get_outline_color (style->marker.mark),
 			go_marker_get_outline_color (default_style->marker.mark),
-			state->gui,
-			"pattern_foreground", "marker_outline_label",
+			gui, "pattern_foreground", "marker_outline_label",
 			G_CALLBACK (cb_marker_outline_color_changed));
 	else {
 		w = create_go_combo_color (state,
 			RGBA_BLUE, RGBA_BLUE,
-			state->gui,
-			"pattern_background", "marker_fill_label",
+			gui, "pattern_background", "marker_fill_label",
 			G_CALLBACK (cb_marker_outline_color_changed));
 		gtk_widget_set_sensitive (w, FALSE);
 	}
 	gtk_table_attach (GTK_TABLE (table), w, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 
-	w = glade_xml_get_widget (state->gui, "marker_size_spin");
+	w = glade_xml_get_widget (gui, "marker_size_spin");
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (w),
 		go_marker_get_size (style->marker.mark));
 	g_signal_connect (G_OBJECT (gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (w))),
@@ -861,6 +836,10 @@ marker_init (StylePrefState *state, gboolean enable)
 		G_CALLBACK (cb_marker_size_changed), state);
 
 	gtk_widget_show_all (table);
+
+	w = glade_xml_get_widget (gui, "gog_style_marker_prefs");
+
+	gog_editor_add_page (editor, w, _("Markers"));
 }
 
 /************************************************************************/
@@ -1020,16 +999,15 @@ gog_style_populate_editor (GogStyle *style,
 	state->object_with_style = object_with_style;
 	state->enable_edit = FALSE;
 
- 	w = glade_xml_get_widget (gui, "gog_style_prefs");
+	w = glade_xml_get_widget (gui, "gog_style_prefs");
 	g_object_set_data_full (G_OBJECT (w),
 		"state", state, (GDestroyNotify) gog_style_pref_state_free);
 	gog_editor_add_page (editor, w, _("Style"));
-	
-	outline_init 	 (state, enable & GOG_STYLE_OUTLINE);
-	line_init    	 (state, enable & GOG_STYLE_LINE);
-	interp_init		 (state, enable & GOG_STYLE_INTERPOLATION);
-	fill_init    	 (state, enable & GOG_STYLE_FILL);
-	marker_init  	 (state, enable & GOG_STYLE_MARKER);
+
+	outline_init 	 (state, enable & GOG_STYLE_OUTLINE, editor);
+	line_init    	 (state, enable & GOG_STYLE_LINE, editor);
+	fill_init    	 (state, enable & GOG_STYLE_FILL, editor);
+	marker_init  	 (state, enable & GOG_STYLE_MARKER, editor, cc);
 	font_init    	 (state, enable & GOG_STYLE_FONT, editor, cc);
 	text_layout_init (state, enable & GOG_STYLE_TEXT_LAYOUT, editor, cc);
 
@@ -1122,7 +1100,6 @@ gog_style_assign (GogStyle *dst, GogStyle const *src)
 	dst->marker = src->marker;
 	dst->marker.mark = go_marker_dup (src->marker.mark);
 	dst->font    = src->font;
-	dst->interpolation    = src->interpolation;
 
 	if (GOG_FILL_STYLE_IMAGE == dst->fill.type)
 		dst->fill.image.filename = g_strdup (dst->fill.image.filename);
@@ -1154,6 +1131,8 @@ gog_style_apply_theme (GogStyle *dst, GogStyle const *src)
 		dst->outline.dash_type = src->outline.dash_type;
 	if (dst->outline.auto_color)
 		dst->outline.color = src->outline.color;
+	if (dst->fill.auto_type)
+		dst->fill.type = src->fill.type;
 	if (dst->fill.auto_fore)
 		dst->fill.pattern.fore = src->fill.pattern.fore;
 	if (dst->fill.auto_back)
@@ -1365,29 +1344,6 @@ gog_style_line_sax_save (GsfXMLOut *output, char const *name,
 	gsf_xml_out_end_element (output);
 }
 
-static void
-gog_style_interpolation_load (xmlNode *node, GogStyle *style)
-{
-	char *str;
-	str = xmlGetProp (node, "type");
-	if (str != NULL) {
-		style->interpolation.type = go_line_interpolation_from_str (str);
-		style->interpolation.auto_type= FALSE;
-		xmlFree (str);
-	}
-}
-
-static void
-gog_style_interpolation_sax_save (GsfXMLOut *output, GogStyle const *style)
-{
-	if (style->interpolation.auto_type)
-		return;
-	gsf_xml_out_start_element (output, "interpolation");
-	gsf_xml_out_add_cstr_unchecked (output, "type",
-		go_line_interpolation_as_str (style->interpolation.type));
-	gsf_xml_out_end_element (output);
-}
-
 static void
 gog_style_gradient_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
@@ -1411,6 +1367,8 @@ gog_style_fill_sax_save (GsfXMLOut *output, GogStyle const *style)
 	gsf_xml_out_start_element (output, "fill");
 	gsf_xml_out_add_cstr_unchecked (output, "type",
 		    fill_style_as_str (style->fill.type));
+	gsf_xml_out_add_bool (output, "auto-type",
+		style->fill.auto_type);
 	gsf_xml_out_add_bool (output, "is-auto",
 		style->fill.auto_back);
 	gsf_xml_out_add_bool (output, "auto-fore",
@@ -1496,6 +1454,10 @@ gog_style_fill_load (xmlNode *node, GogStyle *style)
 	style->fill.type = str_as_fill_style (str);
 	xmlFree (str);
 
+	style->fill.auto_type = FALSE;
+
+	if (bool_prop (node, "auto-type", &tmp))
+		style->fill.auto_type = tmp;
 	if (bool_prop (node, "is-auto", &tmp))
 		style->fill.auto_back = tmp;
 	if (bool_prop (node, "auto-fore", &tmp))
@@ -1639,7 +1601,7 @@ gog_style_font_sax_save (GsfXMLOut *output, GogStyle const *style)
 	gsf_xml_out_start_element (output, "font");
 	go_xml_out_add_color (output, "color", style->font.color);
 	str = go_font_as_str (style->font.font);
-	gsf_xml_out_add_cstr_unchecked (output, "font", str);
+	gsf_xml_out_add_cstr (output, "font", str);
 	g_free (str);
 	gsf_xml_out_add_bool (output, "auto-scale", style->font.auto_scale);
 	gsf_xml_out_end_element (output);
@@ -1679,8 +1641,6 @@ gog_style_persist_dom_load (GogPersist *gp, xmlNode *node)
 			gog_style_line_load (ptr, &style->outline);
 		else if (strcmp (ptr->name, "line") == 0)
 			gog_style_line_load (ptr, &style->line);
-		else if (strcmp (ptr->name, "interpolation") == 0)
-			gog_style_interpolation_load (ptr, style);
 		else if (strcmp (ptr->name, "fill") == 0)
 			gog_style_fill_load (ptr, style);
 		else if (strcmp (ptr->name, "marker") == 0)
@@ -1718,16 +1678,6 @@ gog_style_sax_load_line (GsfXMLIn *xin, xmlChar const **attrs)
 			;
 }
 
-static void
-gog_style_sax_load_interpolation (GsfXMLIn *xin, xmlChar const **attrs)
-{
-	GogStyle *style = GOG_STYLE (xin->user_state);
-	if (0 == strcmp (attrs[0], "type")) {
-		style->interpolation.type = go_line_interpolation_from_str (attrs[1]);
-		style->interpolation.auto_type = FALSE;
-	}
-}
-
 static void
 gog_style_sax_load_fill_pattern (GsfXMLIn *xin, xmlChar const **attrs)
 {
@@ -1775,9 +1725,12 @@ static void
 gog_style_sax_load_fill (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GogStyle *style = GOG_STYLE (xin->user_state);
+	style->fill.auto_type = FALSE;
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
 		if (0 == strcmp (attrs[0], "type"))
 			style->fill.type = str_as_fill_style (attrs[1]);
+		else if (bool_sax_prop ("auto-type", attrs[0], attrs[1], &style->fill.auto_type))
+			;
 		else if (bool_sax_prop ("is-auto", attrs[0], attrs[1], &style->fill.auto_back))
 			;
 		else if (bool_sax_prop ("auto-fore", attrs[0], attrs[1], &style->fill.auto_fore))
@@ -1846,8 +1799,6 @@ gog_style_persist_sax_save (GogPersist const *gp, GsfXMLOut *output)
 		gog_style_line_sax_save (output, "outline", &style->outline);
 	if (style->interesting_fields & GOG_STYLE_LINE)
 		gog_style_line_sax_save (output, "line", &style->line);
-	if (style->interesting_fields & GOG_STYLE_INTERPOLATION)
-		gog_style_interpolation_sax_save (output, style);
 	if (style->interesting_fields & GOG_STYLE_FILL)
 		gog_style_fill_sax_save (output, style);
 	if (style->interesting_fields & GOG_STYLE_MARKER)
@@ -1873,10 +1824,6 @@ gog_style_persist_prep_sax (GogPersist *gp, GsfXMLIn *xin, xmlChar const **attrs
 					 -1, "outline", 
 					 GSF_XML_NO_CONTENT, FALSE, FALSE, 
 					 &gog_style_sax_load_line, NULL, 1),
-		GSF_XML_IN_NODE_FULL 	(STYLE, STYLE_INTERPOLATION, 
-					 -1, "interpolation", 
-					 GSF_XML_NO_CONTENT, FALSE, FALSE, 
-					 &gog_style_sax_load_interpolation, NULL, 0),
 		GSF_XML_IN_NODE 	(STYLE, STYLE_FILL, 
 					 -1, "fill", 
 					 GSF_XML_NO_CONTENT, 
@@ -1943,7 +1890,9 @@ gog_style_is_different_size (GogStyle const *a, GogStyle const *b)
 gboolean
 gog_style_is_marker_visible (GogStyle const *style)
 {
-/* FIXME FIXME FIXME TODO : make this smarter */
+	g_return_val_if_fail (IS_GOG_STYLE (style), FALSE);
+
+	/* FIXME FIXME FIXME TODO : make this smarter */
 	return (style->interesting_fields & GOG_STYLE_MARKER) &&
 		go_marker_get_shape (style->marker.mark) != GO_MARKER_NONE;
 }
@@ -1951,7 +1900,9 @@ gog_style_is_marker_visible (GogStyle const *style)
 gboolean
 gog_style_is_outline_visible (GogStyle const *style)
 {
-/* FIXME FIXME FIXME make this smarter */
+	g_return_val_if_fail (IS_GOG_STYLE (style), FALSE);
+
+	/* FIXME FIXME FIXME make this smarter */
 	return UINT_RGBA_A (style->outline.color) > 0 && 
 		style->outline.dash_type != GO_LINE_NONE;
 }
@@ -1959,14 +1910,25 @@ gog_style_is_outline_visible (GogStyle const *style)
 gboolean
 gog_style_is_line_visible (GogStyle const *style)
 {
-/* FIXME FIXME FIXME TODO : make this smarter */
+	g_return_val_if_fail (IS_GOG_STYLE (style), FALSE);
+
+	/* FIXME FIXME FIXME TODO : make this smarter */
 	return UINT_RGBA_A (style->line.color) > 0 && 
 		style->line.dash_type != GO_LINE_NONE;
 }
 
+gboolean 
+gog_style_is_fill_visible (const GogStyle *style)
+{
+	g_return_val_if_fail (IS_GOG_STYLE (style), FALSE);
+
+	return (style->fill.type != GOG_FILL_STYLE_NONE);
+}
+
 void
 gog_style_force_auto (GogStyle *style)
 {
+	g_return_if_fail (IS_GOG_STYLE (style));
 
 	if (style->marker.mark != NULL)
 		g_object_unref (G_OBJECT (style->marker.mark));
@@ -1978,7 +1940,7 @@ gog_style_force_auto (GogStyle *style)
 	style->outline.auto_color =
 	style->line.auto_dash =
 	style->line.auto_color =
-	style->interpolation.auto_type =
+	style->fill.auto_type =
 	style->fill.auto_fore =
 	style->fill.auto_back =
 	style->font.auto_scale =
@@ -2105,6 +2067,7 @@ gog_style_set_text_angle (GogStyle *style, double angle)
 /**
  * gog_style_create_cairo_pattern:
  * @style : #GogStyle
+ * @cr: a cairo context
  *
  * Create a cairo_patern_t using the current style settings for filling.
  * A pattern will be created only if the style has the corresponding field
@@ -2113,17 +2076,12 @@ gog_style_set_text_angle (GogStyle *style, double angle)
  * return value: the pattern or NULL if it could not be created.
  **/
 cairo_pattern_t *
-gog_style_create_cairo_pattern (GogStyle const *style, double width, double height, gpointer *data)
+gog_style_create_cairo_pattern (GogStyle const *style, cairo_t *cr)
 {
 	cairo_pattern_t *cr_pattern;
-	cairo_surface_t *cr_surface;
 	cairo_matrix_t cr_matrix;
-	GOColor color;
-	GdkPixbuf *pixbuf = NULL;
 	double x[3], y[3];
-	int i, j, w, h;
-	guint8 const *pattern;
-	unsigned char *iter;
+	int w, h;
 
 	static struct { unsigned x0i, y0i, x1i, y1i; } const grad_i[GO_GRADIENT_MAX] = {
 		{0, 0, 0, 1},
@@ -2143,52 +2101,24 @@ gog_style_create_cairo_pattern (GogStyle const *style, double width, double heig
 		{1, 0, 2, 2},
 		{2, 2, 0, 1}
 	};
-	
+
 	g_return_val_if_fail (IS_GOG_STYLE (style), NULL);
 
 	if (style->fill.type == GOG_FILL_STYLE_NONE)
 		return NULL;
 
-	x[0] = y[0] = 0.;
-	x[1] = width;
-	y[1] = height;
-	*data = NULL;
+	cairo_fill_extents (cr, &x[0], &y[0], &x[1], &y[1]);
+	if (go_sub_epsilon (fabs (x[0] - x[1])) <=0.0 ||
+	    go_sub_epsilon (fabs (y[0] - y[1])) <=0.0)
+		return NULL;
+
 	switch (style->fill.type) {
 		case GOG_FILL_STYLE_PATTERN:
-			if (go_pattern_is_solid (&style->fill.pattern, &color))
-				return cairo_pattern_create_rgba (GO_COLOR_TO_CAIRO (color)); 
-			else {
-				GOColor fore = style->fill.pattern.fore;
-				GOColor back = style->fill.pattern.back;
-				int rowstride;
-
-				pattern = go_pattern_get_pattern (&style->fill.pattern);
-				pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 8, 8);
-				*data = pixbuf;
-				iter = gdk_pixbuf_get_pixels (pixbuf);
-				rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-				cr_surface = cairo_image_surface_create_for_data ( iter, 
-					CAIRO_FORMAT_ARGB32, 8, 8, rowstride);
-				for (i = 0; i < 8; i++) {
-					for (j = 0; j < 8; j++) {
-						color = pattern[i] & (1 << j) ? fore : back;
-						iter[0] = UINT_RGBA_B (color);
-						iter[1] = UINT_RGBA_G (color);
-						iter[2] = UINT_RGBA_R (color);
-						iter[3] = UINT_RGBA_A (color);
-						iter += 4;
-					}
-					iter += rowstride - 32;
-				}
-				cr_pattern = cairo_pattern_create_for_surface (cr_surface);
-				cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
-				cairo_surface_destroy (cr_surface);
-				return (cr_pattern);
-			}
+			return go_pattern_create_cairo_pattern (&style->fill.pattern, cr);
 
 		case GOG_FILL_STYLE_GRADIENT:
-			x[2] = (x[1] - x[0]) / 2.0 + x[1];
-			y[2] = (y[1] - y[0]) / 2.0 + y[1];
+			x[2] = (x[1] - x[0]) / 2.0 + x[0];
+			y[2] = (y[1] - y[0]) / 2.0 + y[0];
 			cr_pattern = cairo_pattern_create_linear (
 				x[grad_i[style->fill.gradient.dir].x0i],
 				y[grad_i[style->fill.gradient.dir].y0i],
@@ -2196,28 +2126,28 @@ gog_style_create_cairo_pattern (GogStyle const *style, double width, double heig
 				y[grad_i[style->fill.gradient.dir].y1i]);
 			cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REFLECT);
 			cairo_pattern_add_color_stop_rgba (cr_pattern, 0,
-				GO_COLOR_TO_CAIRO (style->fill.pattern.back)); 
+				GO_COLOR_TO_CAIRO (style->fill.pattern.back));
 			cairo_pattern_add_color_stop_rgba (cr_pattern, 1,
 				GO_COLOR_TO_CAIRO (style->fill.pattern.fore));
-			return (cr_pattern);
+			return cr_pattern;
+
+		case GOG_FILL_STYLE_IMAGE:
+			if (style->fill.image.image == NULL)
+				return cairo_pattern_create_rgba (1, 1, 1, 1);
 
-		case GOG_FILL_STYLE_IMAGE: 
-			if (style->fill.image.image == NULL) {
-				return NULL;
-			}
 			cr_pattern = go_image_create_cairo_pattern (style->fill.image.image);
 			g_object_get (style->fill.image.image, "width", &w, "height", &h, NULL);
 			cairo_pattern_set_extend (cr_pattern, CAIRO_EXTEND_REPEAT);
 			switch (style->fill.image.type) {
 				case GOG_IMAGE_CENTERED:
-					cairo_matrix_init_translate (&cr_matrix, 
-								     -(x[1] - x[0] - w) / 2 - x[0], 
+					cairo_matrix_init_translate (&cr_matrix,
+								     -(x[1] - x[0] - w) / 2 - x[0],
 								     -(y[1] - y[0] - h) / 2 - y[0]);
 					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
 					break;
 				case GOG_IMAGE_STRETCHED:
-					cairo_matrix_init_scale (&cr_matrix, 
-								 w / (x[1] - x[0]), 
+					cairo_matrix_init_scale (&cr_matrix,
+								 w / (x[1] - x[0]),
 								 h / (y[1] - y[0]));
 					cairo_matrix_translate (&cr_matrix, -x[0], -y[0]);
 					cairo_pattern_set_matrix (cr_pattern, &cr_matrix);
