@@ -620,6 +620,13 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 	if (imax ==0 || jmax == 0)
 		return;
 
+	if (plot->plotted_data)
+		data = plot->plotted_data;
+	else
+		data = gog_contour_plot_build_matrix (plot, &cw);
+	if (!data)
+		return;
+
 	x_map = gog_axis_map_new (plot->base.axis[0], 
 				  view->residual.x , view->residual.w);
 	y_map = gog_axis_map_new (plot->base.axis[1], 
@@ -633,11 +640,6 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 		return;
 	}
 
-	if (plot->plotted_data)
-		data = plot->plotted_data;
-	else
-		data = gog_contour_plot_build_matrix (plot, &cw);
-
 	/* Set cw to ensure that polygons will allways be drawn clockwise */
 	xdiscrete = gog_axis_is_discrete (plot->base.axis[0]) ||
 			series->values[(plot->transposed)? 1: 0].data == NULL;
@@ -1127,7 +1129,7 @@ gog_contour_view_render (GogView *view, GogViewAllocation const *bbox)
 							style->fill.pattern.back = color[zn];
 							gog_renderer_push_style (rend, style);
 							gog_renderer_draw_polygon (rend, path, FALSE);
-							if (xl[2] < 0.) {
+							if (xl[4] < 0.) {
 								path[4].x = path[0].x = x[2];
 								path[4].y = path[0].y = y[2];
 								path[5].code = ART_END;
